#!/usr/bin/perl
#
# sendmail-larrd.pl - larrd module for collecting sendmail stats
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com
#
# Christian Pearce
# pearcec@PacketPushers.com
#
#
# Do whatever you want with it, just don't sell it!
# I take no liability for it, use at your own risk!

# V1.0     Scott Walters & Christian Pearce
#
#  1.1 22 Jan 2003    Craig Cook
#      Major rewrite
#      - Uses sub procedures
#      - Added debug output
#
#  1.2 13 Mar 2003    Craig Cook
#      - changed $ENV{'DEBUG'} to $DEBUG

# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
use RRDs;

$ds = "sendmail";

$DEBUG = 1 if $ENV{'DEBUG'};

if ($DEBUG) {
  use Data::Dumper ;  # only for debug output
}

@ds = ("DS:msgsfr:DERIVE:600:0:U",
       "DS:bytes_from:DERIVE:600:0:U",
       "DS:msgsto:DERIVE:600:0:U",
       "DS:bytes_to:DERIVE:600:0:U",
       "DS:msgsrej:DERIVE:600:0:U",
       "DS:msgsdis:DERIVE:600:0:U");

my %h_fn;

##################################
# Start of script
##################################

if( not $ENV{'BBHOME'} ) {
        print "$0: BBHOME is not set\n";
        exit(1);
}

if( not -d $ENV{'BBHOME'} ) {
        print "$0: BBHOME is invalid\n";
        exit(1);
}

if( not -d $ENV{'BBRRDS'} ) {
        print "$0: BBRRDS is not set\n";
        exit(1);
}


##
# real work begins here
##

&read_logs;

print "$0: Data::Dumper->Dump([\%h_fn],[\"h_fn\"])" if $DEBUG ;

while ( ($host,$log) = each %h_fn ) {
    my $new_sm = 0;

    print "$0 : DEBUG host=$host, log=$log\n" if ($DEBUG);

    # 2. Grab the data from the log how ever you need
    &get_info_from_log;

    # Now lets process the list

    &process_log;

    # 3. grab the timestamp for rrd.
   &get_log_time;


  # 4. create the rrd filename

  &update_rrd;

  # delete data log file after we have used it
  unlink $log if $log =~ /$ENV{'BBVAR'}\/data/io;

}

sub read_logs {

  # go to the logs area for simplication of the list
  # Perl Cookbook Recipe 9.5
  opendir(DIR, "$ENV{'BBLOGS'}") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBLOGS'}/$log";
  }
  closedir(DIR);

  opendir(DIR, "$ENV{'BBVAR'}/data") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBVAR'}/data/$log";
  }

  closedir(DIR);

}

sub get_info_from_log {

    print "$0 : DEBUG log=$log\n" if ($DEBUG);

    open(LOG,"$log");

    # Slurp up the log into an array
    @log = <LOG>;

    close(LOG);

}


sub process_log {

  print "$0: DEBUG in process_log\n" if $DEBUG;

  foreach $line (@log) {

    last if ($line =~ /^(purple)/);

    print "$0: DEBUG line: $line\n" if $DEBUG;

    # looking for info like this:
    #
    #Statistics from Wed Jun 19 16:29:41 2002
    # M   msgsfr  bytes_from   msgsto    bytes_to  msgsrej msgsdis  Mailer
    # 3   183435     215701K        0          0K        0       0  local
    # 5        0          0K   183435     215544K        0       0  esmtp
    #=============================================================
    # T   183435     215701K   183435     215544K        0       0
    # C   183435               183435                    0

    next if $line =~ /^Statistics/;
    $new_sm = 1 if ($line =~ /msgsrej msgsdis/);
    next if $line =~ /Mailer$/;
    last if $line =~ /^\=/;
    chomp $line;

    if ($new_sm) {
      ($M,$mfr,$bfr,$mt,$bt,$mr,$md,$mailer) = split " ", $line;
    } else {
      ($M,$mfr,$bfr,$mt,$bt,$mailer) = split " ", $line;
    }

      print "$0: M: $M, mfr: $mfr, bfr: $bfr, mt: $mt, bt: $bt, mr: $mr, md: $md, mailer: $mailer" if ($DEBUG);

  }
}

sub get_log_time {
  # Sometimes you get it from stat, sometimes you get it from the log
  ($ctime) = (stat($log))[10];
  print "$0 : DEBUG ctime = $ctime\n" if ($DEBUG);
}

sub update_rrd {

  $RRD="$ENV{'BBRRDS'}/$host.$ds.$mailer.rrd";

  print "$0 : RRD=$RRD\n" if ($DEBUG);

  # if it doesn't exist, make it
  if ( not -f $RRD ) {
    &create_rrd;
  }

  local($rrdupdate)="$ctime:$mfr:$bfr:$mt:$bt:$mr:$md";

  print "$0 : DEBUG final update: $rrdupdate\n" if ($DEBUG);

  RRDs::update("$RRD","$rrdupdate");
  $ERR=RRDs::error;
  if($ERR) {
    print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
  }
}

sub create_rrd {

  print "$0: DEBUG in create_rrd\n" if $DEBUG;

    # RRAS defined in larrd.pl
    @rras = split " ",$ENV{'RRAS'};

    RRDs::create($RRD,@ds,@rras);
    $ERR=RRDs::error;
    if($ERR) {
      print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
      next;
    }

    $ctime="N";
    print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
}

##############################################
# end of script
##############################################
