#!/bin/sh

# $Id: dns-graph.sh,v 1.3 2003/03/18 16:48:12 cncook Exp $

# set -x

# dns-graph.sh

# this script will build load average PNG's
# using $RRDTOOL graph with RRD's built via
# the BB ext script larrd.sh

if test ! "$BBHOME"
then
        echo "$0: BBHOME is not set"
        exit 1
fi

if test ! -d "$BBHOME"
then
        echo "$0: BBHOME is invalid"
        exit 1
fi

# source the conf
. $BBHOME/etc/bbdef.sh
. $BBHOME/ext/larrd/larrd.conf

# get current date
DATE=`date`

# let's keep the graphs seperate from the RRD's
if [ ! -d $GRAPHS ]
then
	mkdir $GRAPHS	
	echo "$0 did not find $GRAPHS, created." 
fi

# get to work
cd $BBRRDS

for rrd in `ls *dnsdig.rrd`
do
	# need some vars again
	HOST=`echo $rrd | $SED 's/\\.dnsdig\.rrd//' `

	# define the graphs for each time period
	HOURLY=$GRAPHS/$HOST.dnsdig.hourly.png
	DAILY=$GRAPHS/$HOST.dnsdig.daily.png
	WEEKLY=$GRAPHS/$HOST.dnsdig.weekly.png
	MONTHLY=$GRAPHS/$HOST.dnsdig.monthly.png

	# make the hourly graph
	$RRDTOOL graph $HOURLY -s e-48h \
		--title "$HOST Dig Average Last 48 Hours" \
		-z -w 576 -v "dig in milliseconds" -a PNG  \
		DEF:avg=$rrd:dnsdig:AVERAGE AREA:avg#$COLOR:"5 Minute Average" \
		COMMENT:"$DATE\l"


	# make the daily graph
	$RRDTOOL graph $DAILY -s e-12d \
		--title "$HOST Dig Last 12 Days" \
		-w 576 -z -a PNG -v "dig in milliseconds" \
		DEF:avg=$rrd:dnsdig:AVERAGE AREA:avg#$COLOR:"30 Minute Average" \
		COMMENT:"$DATE\l"

	# make the weekly graph
	$RRDTOOL graph $WEEKLY -s e-48d \
		--title "$HOST Dig Last 48 Days" \
		-w 576 -z -a PNG -v "dig in milliseconds" DEF:avg=$rrd:dnsdig:AVERAGE \
		AREA:avg#$COLOR:"2 Hour Average" \
		COMMENT:"$DATE\l"

	# make the monthly graph
	$RRDTOOL graph $MONTHLY -s e-576d \
		--title "$HOST Dig Last 576 Days" \
		-w 576 -z -a PNG -v "dig in milliseconds" DEF:avg=$rrd:dnsdig:AVERAGE \
		AREA:avg#$COLOR:"1 Day Average" \
		COMMENT:"$DATE\l"	

done

# TYPE LIST to change when perlized happens

TYPELIST="dnstotal dnsA dnsMX dnsPTR"

# figure out which hosts have disk info
for rrd in `ls *.dnsstats*` 
do
        HOST=`echo $rrd | $SED 's/\\.dnsdig\.rrd//' `

	# variable reset for each host
	DEFS=""
	LINES=""
	c=0
	p=0
	
	# get list of parition RRD's and build RRD DEF's and LINES
	for type in $TYPELIST 
	do
		# build the DEFs var
		DEFS="$DEFS DEF:$type=$rrd:$type:AVERAGE"

		# get the color to use
		# this is soooooo bad but it was the best 
		# i could come up with
		i=0
		for col in $COLORS
		do
			if [ $c -eq $i ]
			then
				COL=$col
			fi
			i=`expr $i + 1`
		done
		if [ $c -eq 12 ]
		then
			c=0
		else
			c=`expr $c + 1`
		fi
				
		# build the LINES var
		LINES="$LINES LINE2:$type#$COL:$type"
	done

	# now start building graphs
	# define the graphs for each time period
	HOURLY=$GRAPHS/$HOST.dnsstats.hourly.png
	DAILY=$GRAPHS/$HOST.dnsstats.daily.png
	WEEKLY=$GRAPHS/$HOST.dnsstats.weekly.png
	MONTHLY=$GRAPHS/$HOST.dnsstats.monthly.png

	# make the hourly graph
	$RRDTOOL graph $HOURLY -s e-48h --title="$HOST DNS last 48 Hours" \
	-w 576 -a PNG -z -v "queries/second"  $DEFS $LINES \
	COMMENT:"\s" COMMENT:"$DATE\r"
	
	# make the daily graph
	$RRDTOOL graph $DAILY -s e-12d --title="$HOST DNS last 12 Days" \
	-w 576 -a PNG -z -v "queries/second" $DEFS $LINES \
	COMMENT:"\s" COMMENT:"$DATE\r"

	# make the weekly graph
 	$RRDTOOL graph $WEEKLY -s e-48d --title="$HOST DNS last 48 Days" \
	-w 576 -a PNG -z -v "queries/second" $DEFS $LINES COMMENT:"\s" \
	COMMENT:"$DATE\r"

	# make the monthly 
	$RRDTOOL graph $MONTHLY -s e-576d --title="$HOST DNS last 576 Days" \
	-w 576 -a PNG -z -v "queries/second" $DEFS $LINES COMMENT:"\s" \
	COMMENT:"$DATE\r"

done

