#!/usr/bin/perl

# $Id: larrd-grapher.cgi,v 1.2 2003/03/18 16:48:12 cncook Exp $

# larrd-grapher.cgi:  The larrd cgi script to dynamically generate graphs.

# Scott Walters Copyright 1999
# scott@PacketPushers.com
#
# Christian Pearce
# pearcec@PacketPushers.com
#
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks
#
# Version 1.0    Original  by Scott Walters
#         1.1    12 Dec 2002   Craig Cook
#                - Added nmailq, mailq, socks
#                - lots of bug fixes
#         1.2    14 Jan 2003   Tom Schmidt
#                - Added temperature
#                - lots of cleanup and bug fixes
#         1.3    14 Jan 2003   Craig Cook
#                - Added disk usage graph
#                - Added patch by Jason Kruse
#                  - Now only 5 filesystems per disk graph


use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser set_message);
BEGIN {
    sub handle_errors {
        my $msg = shift;
        print "$0: $msg";
    }
    set_message(\&handle_errors);
}


use lib qq{/opt/larrd};
require "larrd-config.pl";

use lib qq{$ENV{RRDPM}};
use RRDs;

if(not -d $ENV{'LARRDTMP'}) {
    mkdir "$ENV{'LARRDTMP'}",0755;
}

my $DEBUG = 0;

$DEBUG = 1 if $ENV{'DEBUG'};

open(TMP,">>$ENV{'LARRDTMP'}/larrd-grapher.debug") if $DEBUG;

use vars qw/$p/;

my($q) = new CGI;

my($cgiurl) = $q->url;

my($host) = $q->param('host') or die "Could not find host parameter";

# Local allows for dynamically scoped values.  IE the refs in the graph_c hash
# sloppy but works for now.  Should make a sub return defs and lines.

my($service) = $q->param('service') or die "Could not find service parameter";
my($graph) = $q->param('graph');
my($disk_part) = $q->param('disk_part');
my $date = localtime;

print TMP "service: $service\n"     if $DEBUG;
print TMP "graph: $graph\n"         if $DEBUG;
print TMP "disk_part: $disk_part\n" if $DEBUG;

my $single = "";


# TODO use come config;

my(%graph_c) = (

  nmailq => {
    hourly => {
	start_sec => "e-48h",
	title => "Messages in Mail Queues",
	legend => "5 Minute average"
    },
    daily => {
        start_sec => "e-12d",
        title => "Messages in Mail Queues Last 12 Days",
        legend => "30 Minute Average"
    },
    weekly => {
        start_sec => "e-48d",
        title => "Messages in Mail Queues Last 48 Days",
        legend => "2 Hour Average"
    },
    monthly => {
        start_sec => "e-576d",
        title => "Messages in Mail Queues Last 576 Days",
        legend => "1 Day Average"
    },
    yaxis => "Messages",
    lines => sub {
        my ($RRD,$service,$graph)=@_;
	my ($type);
	my $counter=0;
        my @colors=("#0000FF","#00FFFF","#00FF00","#A0522D","#FFA500","#FF7F50","#FF0000","#FF00FF");

	# Get the data values from the dataset
	my ($start,$step,$names,$data)=RRDs::fetch($RRD,'AVERAGE','-s',0,'-e',0);
	my @ds = @$names;
	# Build cdef total value
	# There is probably a better way to do this, but I can't come up with it right now.
	my $cdef = "CDEF:Total=". join (",", @ds);
        my $i;
	until ($i > $#ds - 1) {
	  $cdef .= ",+";
	  $i++
	}

	my @t_def;

	foreach my $ds (@ds) {
          my $color = $counter % $#colors;
	  if( $counter == 0) {
	    $type = "AREA";
	  } else {
	    $type = "STACK";
	  }
	  ++$counter;
	  push @t_def, qq{DEF:$ds=$RRD:$ds:AVERAGE};
	  push @t_def, qq{$type:$ds$colors[$color]:$ds};
	  push @t_def, qq{GPRINT:$ds:LAST: %.0lf (cur)};
	  push @t_def, qq{GPRINT:$ds:MAX: %.0lf (max)};
  	  push @t_def, qq{GPRINT:$ds:MIN: %.0lf (min)};
	  push @t_def, qq{GPRINT:$ds:AVERAGE: %.0lf (avg)\\n};
	
	}
	push @t_def, qq{$cdef};
	push @t_def, qq{LINE2:Total#000000:Total Messages};
	push @t_def, qq{GPRINT:Total:LAST:\\: %.0lf};
	push @t_def, qq{COMMENT:\\n};
        return \@t_def;
    },
  },

  mailq => {
    hourly => {
        start_sec => "e-48h",
        title => "Mail Queue Length",
        legend => "5 Minute average"
    },
    daily => {
        start_sec => "e-12d",
        title => "Mail Queue Length Last 12 Days",
        legend => "30 Minute Average"
    },
    weekly => {
        start_sec => "e-48d",
        title => "Mail Queue Length Last 48 Days",
        legend => "2 Hour Average"
    },
    monthly => {
        start_sec => "e-576d",
        title => "Mail Queue Length Last 576 Days",
        legend => "1 Day Average"
    },
    yaxis => "Queue Length",
    lines => sub {
        my ($RRD,$service,$graph)=@_;
        return [
            qq{DEF:mailq=$RRD:$service:AVERAGE},
            qq{AREA:mailq#00CCCC:Queue Length},
            qq{COMMENT:\\n},
            qq{GPRINT:mailq:LAST: \\: %5.1lf (cur)},
            qq{GPRINT:mailq:MAX: \\: %5.1lf (max)},
            qq{GPRINT:mailq:MIN: \\: %5.1lf (min)},
            qq{GPRINT:mailq:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  imap2 => {
    hourly => {
        start_sec => "e-48h",
        title => "IMAP Response Time",
        legend => "5 Minute average"
    },
    daily => {
        start_sec => "e-12d",
        title => "IMAP Response Time Last 12 Days",
        legend => "30 Minute Average"
    },
    weekly => {
        start_sec => "e-48d",
        title => "IMAP Response Time Last 48 Days",
        legend => "2 Hour Average"
    },
    monthly => {
        start_sec => "e-576d",
        title => "IMAP Response Time Last 576 Days",
        legend => "1 Day Average"
    },
    yaxis => "Seconds",
    lines => sub {
        my ($RRD,$service,$graph)=@_;
        return [
            qq{DEF:imap2=$RRD:$service:AVERAGE},
            qq{AREA:imap2#00CCCC:Response Time},
            qq{COMMENT:\\n},
            qq{GPRINT:imap2:LAST: \\: %5.1lf (cur)},
            qq{GPRINT:imap2:MAX: \\: %5.1lf (max)},
            qq{GPRINT:imap2:MIN: \\: %5.1lf (min)},
            qq{GPRINT:imap2:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  socks => {
    hourly => {
        start_sec => "e-48h",
        title => "Open Sockets Threads",
        legend => "5 Minute average"
    },
    daily => {
        start_sec => "e-12d",
        title => "Open Sockets Threads Last 12 Days",
        legend => "30 Minute Average"
    },
    weekly => {
        start_sec => "e-48d",
        title => "Open Sockets Threads Last 48 Days",
        legend => "2 Hour Average"
    },
    monthly => {
        start_sec => "e-576d",
        title => "Open Sockets Threads Last 576 Days",
        legend => "1 Day Average"
    },
    yaxis => "Open Threads",
    lines => sub {
        my ($RRD,$service,$graph)=@_;
        return [
            qq{DEF:socks=$RRD:$service:AVERAGE},
            qq{CDEF:th=socks,2,-},
            qq{AREA:th#9400D3:Open Threads},
            qq{COMMENT:\\n},
            qq{GPRINT:th:LAST: \\: %5.1lf (cur)},
            qq{GPRINT:th:MAX: \\: %5.1lf (max)},
            qq{GPRINT:th:MIN: \\: %5.1lf (min)},
            qq{GPRINT:th:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  la => {
    hourly => {
        start_sec => "e-48h",
        title => "$host CPU Load Average Last 48 Hours",
        legend => "5 Minute average"
    },
    daily => {
        start_sec => "e-12d",
        title => "$host CPU Load Average Last 12 Days",
        legend => "30 Minute Average"
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host CPU Load Average Last 48 Days",
        legend => "2 Hour Average"
    },
    monthly => {
        start_sec => "e-576d",
        title => "$host CPU Load Average Last 576 Days",
        legend => "1 Day Average"
    },
    yaxis => "CPU Load Average",
    lines => sub {
        my ($RRD,$service,$graph)=@_;
        my @t_def = (
            qq{DEF:avg=$RRD:$service:AVERAGE},
            qq{CDEF:la=avg,100,/},
            qq{AREA:la#00CC00:CPU Load Average},
            qq{GPRINT:la:LAST: \\: %5.1lf (cur)},
            qq{GPRINT:la:MAX: \\: %5.1lf (max)},
            qq{GPRINT:la:MIN: \\: %5.1lf (min)},
            qq{GPRINT:la:AVERAGE: \\: %5.1lf (avg)\\n},
        );
	push @t_def, q{-u 1.0} if ($ENV{'LARRDLARIGID'});
	return \@t_def;
    },
  },

  disk => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Disk Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Disk Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Disk Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Disk Utilization Last 576 Days",
    },
    yaxis => "% Full",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/^.*disk,/,/;
        $part =~ s/^.*diskswap\.rrd/swap/;    # swap space from ext/dspace.sh
        $part =~ s/\.rrd//;
        $part =~ s/\,/\//g;

        $part = "/" if ($part eq "/root");

        # Pad with spaces or line feed for GPRINT below
        my $padding = '                  ';
        my $comment = '\\n';
        if (length($part) <= 14) {
            $padding = substr($padding,0,14-length($part));
            $comment = '';
        }

        return [
            qq{DEF:$p=$RRD:pct:AVERAGE},
            qq{LINE2:$p#$color:$part},
            qq{-u 100},
            qq{COMMENT:$comment},
            qq{GPRINT:$p:LAST:$padding \\: %5.1lf (cur)},
            qq{GPRINT:$p:MAX: \\: %5.1lf (max)},
            qq{GPRINT:$p:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  disk1 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Disk Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Disk Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Disk Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Disk Utilization Last 576 Days",
    },
    yaxis => "Total Used",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/^.*disk,/,/;
        $part =~ s/^.*diskswap\.rrd/swap/;    # swap space from ext/dspace.sh
        $part =~ s/\.rrd//;
        $part =~ s/\,/\//g;

        $part = "/" if ($part eq "/root");

        # Pad with spaces or line feed for GPRINT below
        my $padding = '                  ';
        my $comment = '\\n';
        if (length($part) <= 14) {
            $padding = substr($padding,0,14-length($part));
            $comment = '';
        }

        return [
            qq{DEF:$p=$RRD:used:AVERAGE},
            qq{LINE2:$p#$color:$part},
            qq{COMMENT:$comment},
            qq{GPRINT:$p:LAST:$padding \\: %5.1lf%s (cur)},
            qq{GPRINT:$p:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:$p:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:$p:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  tcp => {
    hourly => {
        start_sec => "e-48h",
        title => "$host TCP Connection Times Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host TCP Connection Times Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host TCP Connection Times Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host TCP Connection Times Last 576 Days",
    },
    yaxis => "Seconds",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        $RRD =~ s/:/\\:/g;
        my($part) = $RRD;

        $part =~ s/^.*tcp./,/;
        $part =~ s/\.rrd//;
        $part =~ s/\,/\//g;
        $part =~ s/^\///g;
        # Make this look like a http://
        $part =~ s/http\./http\\\:\/\//g;

        # Pad with spaces or line feed for GPRINT below
        my $padding = '                  ';
        my $comment = '\\n';
        if (length($part) <= 14) {
            $padding = substr($padding,0,14-length($part));
            $comment = '';
        }

        return [
            qq{DEF:$p=$RRD:sec:AVERAGE},
            qq{LINE2:$p#$color:$part},
            qq{COMMENT:$comment},
            qq{GPRINT:$p:LAST:$padding \\: %5.1lf%s (cur)},
            qq{GPRINT:$p:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:$p:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:$p:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat => {
    hourly => {
        start_sec => "e-48h",
        title => "$host CPU Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host CPU Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host CPU Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host CPU Utilization Last 576 Days",
    },
    yaxis => "% Load",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{-u 100},qq{-r},
            qq{DEF:cpu_idl=$RRD:cpu_idl:AVERAGE},
            qq{DEF:cpu_usr=$RRD:cpu_usr:AVERAGE},
            qq{DEF:cpu_sys=$RRD:cpu_sys:AVERAGE},
            qq{AREA:cpu_sys#FF0000:System},
            qq{STACK:cpu_usr#FFFF00:User},
            qq{STACK:cpu_idl#00FF00:Idle},
            qq{COMMENT:\\n},
            qq{GPRINT:cpu_sys:LAST:System  \\: %5.1lf (cur)},
            qq{GPRINT:cpu_sys:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_sys:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_sys:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:cpu_usr:LAST:User    \\: %5.1lf (cur)},
            qq{GPRINT:cpu_usr:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_usr:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_usr:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:cpu_idl:LAST:Idle    \\: %5.1lf (cur)},
            qq{GPRINT:cpu_idl:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_idl:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_idl:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  vmstat2 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host CPU Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host CPU Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host CPU Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host CPU Utilization Last 576 Days",
    },
    yaxis => "CPU Load Average",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{-u 1.0},

            qq{DEF:avg=$host.la.rrd:la:AVERAGE},
            qq{CDEF:la=avg,100,/},

            qq{DEF:cpu_idl=$RRD:cpu_idl:AVERAGE},
            qq{CDEF:cpu_idl2=cpu_idl,100,/},

            qq{DEF:cpu_usr=$RRD:cpu_usr:AVERAGE},
            qq{CDEF:cpu_usr2=cpu_usr,100,/},

	    qq{DEF:cpu_sys=$RRD:cpu_sys:AVERAGE},
	    qq{CDEF:cpu_sys2=cpu_sys,100,/},

	    qq{AREA:cpu_sys2#FF0000:System},
            qq{STACK:cpu_usr2#FFFF00:User},
            qq{STACK:cpu_idl2#00FF00:Idle},
	    qq{LINE1:la#000099:CPU Load Average},
            qq{COMMENT:\\n},
            qq{GPRINT:cpu_sys2:LAST:System           \\: %5.1lf (cur)},
            qq{GPRINT:cpu_sys2:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_sys2:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_sys2:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:cpu_usr2:LAST:User             \\: %5.1lf (cur)},
            qq{GPRINT:cpu_usr2:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_usr2:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_usr2:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:cpu_idl2:LAST:Idle             \\: %5.1lf (cur)},
            qq{GPRINT:cpu_idl2:MAX: \\: %5.1lf (max)},
            qq{GPRINT:cpu_idl2:MIN: \\: %5.1lf (min)},
            qq{GPRINT:cpu_idl2:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:la:LAST:CPU Load Average \\: %5.1lf (cur)},
            qq{GPRINT:la:MAX: \\: %5.1lf (max)},
            qq{GPRINT:la:MIN: \\: %5.1lf (min)},
            qq{GPRINT:la:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  vmstat3 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host vmstat Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host vmstat Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host vmstat Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host vmstat Last 576 Days",
    },
    yaxis => "Average Events/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            #qq{-u 1.0},

            qq{DEF:int=$host.vmstat.rrd:cpu_int:AVERAGE},
            qq{DEF:csw=$host.vmstat.rrd:cpu_csw:AVERAGE},

            qq{LINE2:int#0000ff:Interrupts},
            qq{LINE2:csw#00FF00:Context Switches},
            qq{COMMENT:\\n},
            qq{GPRINT:int:LAST:Interrupts        \\: %5.1lf%s (cur)},
            qq{GPRINT:int:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:int:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:int:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:csw:LAST:Context Switches  \\: %5.1lf%s (cur)},
            qq{GPRINT:csw:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:csw:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:csw:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat4 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host vmstat Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host vmstat Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host vmstat Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host vmstat 576 Days",
    },
    yaxis => "Average kB/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{-b 1024},

            qq{DEF:si=$host.vmstat.rrd:mem_si:AVERAGE},
            qq{DEF:so=$host.vmstat.rrd:mem_so:AVERAGE},

            qq{LINE2:si#0000ff:Memory Swapped from Disk},
            qq{LINE2:so#00FF00:Memory Swapped to Disk},
            qq{COMMENT:\\n},
            qq{GPRINT:si:LAST:Memory Swapped from Disk  \\: %5.1lf (cur)},
            qq{GPRINT:si:MAX: \\: %5.1lf (max)},
            qq{GPRINT:si:MIN: \\: %5.1lf (min)},
            qq{GPRINT:si:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:si:LAST:Memory Swapped to Disk    \\: %5.1lf (cur)},
            qq{GPRINT:si:MAX: \\: %5.1lf (max)},
            qq{GPRINT:si:MIN: \\: %5.1lf (min)},
            qq{GPRINT:si:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat5 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host vmstat Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host vmstat Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host vmstat Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host vmstat 576 Days",
    },
    yaxis => "Average blocks/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            #qq{-b 1024},

            qq{DEF:bi=$host.vmstat.rrd:dsk_bi:AVERAGE},
            qq{DEF:bo=$host.vmstat.rrd:dsk_bo:AVERAGE},

            qq{LINE2:bi#0000FF:Blocks sent to a block device},
            qq{LINE2:bo#00FF00:Blocks received from a block device},
            qq{COMMENT:\\n},
            qq{GPRINT:bi:LAST:Blocks sent to a block device        \\: %5.1lf (cur)},
            qq{GPRINT:bi:MAX: \\: %5.1lf (max)},
            qq{GPRINT:bi:MIN: \\: %5.1lf (min)},
            qq{GPRINT:bi:AVERAGE: \\: %5.1lf (avg)\\n},
            qq{GPRINT:bo:LAST:Blocks received from a block device  \\: %5.1lf (cur)},
            qq{GPRINT:bo:MAX: \\: %5.1lf (max)},
            qq{GPRINT:bo:MIN: \\: %5.1lf (min)},
            qq{GPRINT:bo:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  vmstat6 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Memory Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Memory Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Memory Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Memory Utilization Last 576 Days",
    },
    yaxis => "Size (1K=1024)",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{-b 1024},

            qq{DEF:free=$host.vmstat.rrd:mem_free:AVERAGE},
            qq{CDEF:fr=free,1024,*},
            qq{DEF:swap=$host.vmstat.rrd:mem_swap:AVERAGE},
            qq{CDEF:sw=swap,1024,*},

            qq{LINE2:fr#0000ff:Size of the free list},
            qq{LINE2:sw#00FF00:Swap space currently available},
            qq{COMMENT:\\n},
            qq{GPRINT:fr:LAST:Size of the free list           \\: %5.1lf%s (cur)},
            qq{GPRINT:fr:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:fr:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:fr:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:sw:LAST:Swap space currently available  \\: %5.1lf%s (cur)},
            qq{GPRINT:sw:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:sw:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:sw:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat7 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Memory Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Memory Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Memory Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Memory Utilization Last 576 Days",
    },
    yaxis => "Events/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            #qq{-b 1024},

            qq{DEF:re=$host.vmstat.rrd:mem_re:AVERAGE},
            qq{DEF:mf=$host.vmstat.rrd:mem_mf:AVERAGE},

            qq{LINE2:re#0000ff:Page reclaims},
            qq{LINE2:mf#00FF00:Minor faults},
            qq{COMMENT:\\n},
            qq{GPRINT:re:LAST:Page reclaims  \\: %5.1lf%s (cur)},
            qq{GPRINT:re:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:re:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:re:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:mf:LAST:Minor faults   \\: %5.1lf%s (cur)},
            qq{GPRINT:mf:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:mf:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:mf:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat8 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Paging Activity Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Paging Activity Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Paging Activity Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Paging Activity Last 576 Days",
    },
    yaxis => "K/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{-b 1024},

            qq{DEF:pi=$host.vmstat.rrd:mem_pi:AVERAGE},
            qq{CDEF:rpi=pi,1024,*},
            qq{DEF:po=$host.vmstat.rrd:mem_po:AVERAGE},
            qq{CDEF:rpo=po,1024,*},

            qq{LINE2:rpi#0000ff:Paged in},
            qq{LINE2:rpo#00FF00:Paged out},
            qq{COMMENT:\\n},
            qq{GPRINT:pi:LAST:Page Ins   \\: %5.1lf%s (cur)},
            qq{GPRINT:pi:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:pi:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:pi:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:po:LAST:Page Outs  \\: %5.1lf%s (cur)},
            qq{GPRINT:po:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:po:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:po:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  vmstat9 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host RAM Scan Rate Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host RAM Scan Rate Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host RAM Scan Rate Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host RAM Scan Rate Last 576 Days",
    },
    yaxis => "Pages Scanned/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:sr=$host.vmstat.rrd:sr:AVERAGE},
            qq{AREA:sr#00CC00:Scan Rate},
            qq{-u 20},
            qq{GPRINT:sr:LAST: \\: %5.1lf%s (cur)},
            qq{GPRINT:sr:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:sr:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:sr:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
        },
  },

  vmstat0 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host System Calls Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host System Calls Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host System Calls Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host System Calls Last 576 Days",
    },
    yaxis => "System Calls/s",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:cpu_syc=$host.vmstat.rrd:cpu_syc:AVERAGE},
            qq{AREA:cpu_syc#00CC00:System Calls},
            #qq{-u 20},
            qq{GPRINT:cpu_syc:LAST: \\: %5.1lf%s (cur)},
            qq{GPRINT:cpu_syc:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:cpu_syc:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:cpu_syc:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

#memory added by JG 11-1-2000 <josh@techsurgeons.com>
  memory => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Memory Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Memory Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Memory Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Memory Utilization Last 576 Days",
    },
    yaxis => "% Full",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/\,/\./g;
        $part =~ s/^.*memory\.//;
        $part =~ s/\.rrd//;

        return [
            qq{DEF:$p=$RRD:realmempct:AVERAGE},
            qq{LINE2:$p#$color:$part},
            qq{-u 100}, qq{-b 1024},
            qq{GPRINT:$p:LAST: \\: %5.1lf (cur)},
            qq{GPRINT:$p:MAX: \\: %5.1lf (max)},
            qq{GPRINT:$p:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

  bind => {
    hourly => {
        start_sec => "e-48h",
        title => "$host BIND Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host BIND Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host BIND Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host BIND Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:A=$RRD:A:AVERAGE},
            qq{DEF:ANY=$RRD:ANY:AVERAGE},
            qq{DEF:AXFR=$RRD:AXFR:AVERAGE},
            qq{DEF:MX=$RRD:MX:AVERAGE},
            qq{DEF:O=$RRD:O:AVERAGE},
            qq{DEF:PTR=$RRD:PTR:AVERAGE},
            qq{DEF:SOA=$RRD:SOA:AVERAGE},
            qq{CDEF:A1=A,300,*},
            qq{CDEF:ANY1=ANY,300,*},
            qq{CDEF:AXFR1=AXFR,300,*},
            qq{CDEF:MX1=MX,300,*},
            qq{CDEF:O1=O,300,*},
            qq{CDEF:PTR1=PTR,300,*},
            qq{CDEF:SOA1=SOA,300,*},
            qq{LINE2:A1#0000FF:A},
            qq{LINE2:ANY1#FF0000:ANY},
            qq{LINE2:AXFR1#00CC00:AXFR},
            qq{LINE2:MX1#FF00FF:MX},
            qq{LINE2:O1#555555:O},
            qq{LINE2:PTR1#880000:PTR},
            qq{LINE2:SOA1#000088:SOA},
            qq{COMMENT:\\n},
            qq{GPRINT:A1:LAST:A     \\: %5.1lf (cur) },
            qq{GPRINT:A1:MAX:A    \\: %5.1lf (max) },
            qq{GPRINT:A1:MIN:A     \\: %5.1lf (min) },
            qq{GPRINT:A1:AVERAGE:A     \\: %5.1lf (avg)\\n},
            qq{GPRINT:ANY1:LAST:ANY   \\: %5.1lf (cur) },
            qq{GPRINT:ANY1:MAX:ANY  \\: %5.1lf (max) },
            qq{GPRINT:ANY1:MIN:ANY   \\: %5.1lf (min) },
            qq{GPRINT:ANY1:AVERAGE:ANY   \\: %5.1lf (avg)\\n},
            qq{GPRINT:AXFR1:LAST:AXFR  \\: %5.1lf (cur) },
            qq{GPRINT:AXFR1:MAX:AXFR \\: %5.1lf (max) },
            qq{GPRINT:AXFR1:MIN:AXFR  \\: %5.1lf (min) },
            qq{GPRINT:AXFR1:AVERAGE:AXFR  \\: %5.1lf (avg)\\n},
            qq{GPRINT:MX1:LAST:MX    \\: %5.1lf (cur) },
            qq{GPRINT:MX1:MAX:MX   \\: %5.1lf (max) },
            qq{GPRINT:MX1:MIN:MX    \\: %5.1lf (min) },
            qq{GPRINT:MX1:AVERAGE:MX    \\: %5.1lf (avg)\\n},
            qq{GPRINT:O1:LAST:O     \\: %5.1lf (cur) },
            qq{GPRINT:O1:MAX:O    \\: %5.1lf (max) },
            qq{GPRINT:O1:MIN:O     \\: %5.1lf (min) },
            qq{GPRINT:O1:AVERAGE:O     \\: %5.1lf (avg)\\n},
            qq{GPRINT:PTR1:LAST:PTR   \\: %5.1lf (cur) },
            qq{GPRINT:PTR1:MAX:PTR  \\: %5.1lf (max) },
            qq{GPRINT:PTR1:MIN:PTR   \\: %5.1lf (min) },
            qq{GPRINT:PTR1:AVERAGE:PTR   \\: %5.1lf (avg)\\n},
            qq{GPRINT:SOA1:LAST:SOA   \\: %5.1lf (cur) },
            qq{GPRINT:SOA1:MAX:SOA  \\: %5.1lf (max) },
            qq{GPRINT:SOA1:MIN:SOA   \\: %5.1lf (min) },
            qq{GPRINT:SOA1:AVERAGE:SOA   \\: %5.1lf (avg)\\n},
        ]
    },
  },

  apache => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Apache Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Apache Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Apache Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Apache Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:TA=$RRD:TA:AVERAGE},
	    qq{CDEF:TA1=TA,300,*},
            qq{DEF:TKB=$RRD:TKB:AVERAGE},
	    qq{CDEF:TKB1=TKB,300,*},
            qq{LINE2:TA1#0000FF:TA (# Accesses)},
            qq{LINE2:TKB1#FF0000:TKB (kBytes Transferred)},

            qq{COMMENT:\\n},

            qq{GPRINT:TA1:LAST:TA    \\: %5.1lf%s (cur)},
            qq{GPRINT:TA1:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:TA1:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:TA1:AVERAGE: \\: %5.1lf%s (avg)\\n},

            qq{GPRINT:TKB1:LAST:TKB   \\: %5.1lf%s (cur)},
            qq{GPRINT:TKB1:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:TKB1:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:TKB1:AVERAGE: \\: %5.1lf%s (avg)\\n},

        ]
    },
  },


  apache1 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Apache Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Apache Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Apache Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Apache Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
        #    qq{DEF:BS=$RRD:BS:AVERAGE},
        #    qq{DEF:IS=$RRD:IS:AVERAGE},
            qq{DEF:BW=$RRD:BW:AVERAGE},
            qq{DEF:IW=$RRD:IW:AVERAGE},
        #    qq{LINE2:BS#00CC00:BS},
        #    qq{LINE2:IS#FF00FF:IS},
            qq{LINE2:BW#555555:BW Busy Workers},
            qq{LINE2:IW#880000:IW Idle Workers},

            qq{COMMENT:\\n},

            qq{GPRINT:BW:LAST:BW   \\: %5.1lf%s (cur)},
            qq{GPRINT:BW:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:BW:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:BW:AVERAGE: \\: %5.1lf%s (avg)\\n},

            qq{GPRINT:IW:LAST:IW   \\: %5.1lf%s (cur)},
            qq{GPRINT:IW:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:IW:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:IW:AVERAGE: \\: %5.1lf%s (avg)\\n},

        ]
    },
  },


  apache2 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Apache CPU Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Apache CPU Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Apache CPU Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Apache CPU Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:CPU=$RRD:CPU:AVERAGE},
            qq{LINE2:CPU#000088:CPU},

            qq{COMMENT:\\n},

            qq{GPRINT:CPU:LAST:CPU   \\: %5.1lf (cur)},
            qq{GPRINT:CPU:MAX: \\: %5.1lf (max)},
            qq{GPRINT:CPU:MIN: \\: %5.1lf (min)},
            qq{GPRINT:CPU:AVERAGE: \\: %5.1lf (avg)\\n},

        ]
    },
  },

  apache3 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Apache Requests Per Second Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Apache Requests Per Second Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Apache Requests Per Second Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Apache Requests Per Second Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:RPS=$RRD:REQPERSEC:AVERAGE},
            qq{LINE2:RPS#000088:RPS},

            qq{COMMENT:\\n},

            qq{GPRINT:RPS:LAST:Requests per Second   \\: %5.1lf (cur)},
            qq{GPRINT:RPS:MAX: \\: %5.1lf (max)},
            qq{GPRINT:RPS:MIN: \\: %5.1lf (min)},
            qq{GPRINT:RPS:AVERAGE: \\: %5.1lf (avg)\\n},

        ]
    },
  },

  iostat => {
    hourly => {
        start_sec => "e-48h",
        title => "$host IO Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host IO Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host IO Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host IO Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/^.*iostat.,/,/;
        $part =~ s/\.rrd//;
        $part =~ s/\,/\//g;
	# this was not used; cnt was not defined
	#return if $cnt++ > 10;

        return [
            qq{DEF:$p}   . qq{B=$RRD:b:AVERAGE},
            qq{DEF:$p}   . qq{A=$RRD:asvc_t:AVERAGE},
            qq{DEF:$p}   . qq{W=$RRD:wsvc_t:AVERAGE},
            #This doesn't work, need to work out what it is trying to do
            #qq{CDEF:$p}  . qq{t=B,5,GT,A,W,+,UNKN,IF},

            #qq{CDEF:$p}  . qq{t=$p\B,5,GT,$p\A,$p\W,+,UNKN,IF},
            qq{CDEF:$p}  . qq{t=$p\B,5,GT,$p\B,UNKN,IF},
            qq{LINE2:$p} . qq{t#} . &color() . qq{:$part},

            qq{COMMENT:\\n},
            qq{GPRINT:$p} . qq{B:LAST:B       \\: %5.1lf (cur)},
            qq{GPRINT:$p} . qq{B:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p} . qq{B:AVERAGE: \\: %5.1lf (avg)},
            qq{GPRINT:$p} . qq{B:MAX: \\: %5.1lf (max)\\n},

            qq{GPRINT:$p} . qq{A:LAST:asvc_t  \\: %5.1lf (cur)},
            qq{GPRINT:$p} . qq{A:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p} . qq{A:AVERAGE: \\: %5.1lf (avg)},
            qq{GPRINT:$p} . qq{A:MAX: \\: %5.1lf (max)\\n},

            qq{GPRINT:$p} . qq{W:LAST:wsvc_t  \\: %5.1lf (cur)},
            qq{GPRINT:$p} . qq{W:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p} . qq{W:AVERAGE: \\: %5.1lf (avg)},
            qq{GPRINT:$p} . qq{W:MAX: \\: %5.1lf (max)\\n},
	]
    },
  },

  netstat2 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Net Statistics Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host NET Statistics Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host NET Statistics Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host NET Statistics Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:tcpActiveOpens=$RRD:tcpActiveOpens:AVERAGE},
            qq{DEF:tcpPassiveOpens=$RRD:tcpPassiveOpens:AVERAGE},
            qq{DEF:tcpAttemptFails=$RRD:tcpAttemptFails:AVERAGE},
            qq{DEF:tcpEstabResets=$RRD:tcpEstabResets:AVERAGE},
            #qq{DEF:tcpCurrEstab=$RRD:tcpCurrEstab:AVERAGE},
#            qq{CDEF:tcpCurrEstab1=tcpCurrEstab,300,*},
            qq{LINE2:tcpActiveOpens#0000FF:TCP Active Open},
            qq{LINE2:tcpPassiveOpens#000088:TCP Passive Open},
            qq{LINE2:tcpAttemptFails#FF0000:TCP Attempts Fail},
            qq{LINE2:tcpEstabResets#88FF00:TCP Established Resets},
# This one spikes the graph
            #qq{LINE2:tcpCurrEstab#00FF00:TCP Established Connections},
            qq{COMMENT:\\n},
            qq{GPRINT:tcpActiveOpens:LAST:TCP Active Opens            \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpActiveOpens:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpActiveOpens:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpActiveOpens:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:tcpPassiveOpens:LAST:TCP Passive Opens           \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpPassiveOpens:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpPassiveOpens:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpPassiveOpens:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:tcpAttemptFails:LAST:TCP Failed Attempts         \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpAttemptFails:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpAttemptFails:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpAttemptFails:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:tcpEstabResets:LAST:TCP Established Resets      \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpEstabResets:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpEstabResets:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpEstabResets:AVERAGE: \\: %5.1lf%s (avg)\\n},
            #qq{GPRINT:tcpCurrEstab:MAX:TCP Established Connections \\: %5.1lf%s (max)\\n},
            #qq{GPRINT:tcpCurrEstab:MIN:TCP Established Connections  \\: %5.1lf%s (min)\\n},
            #qq{GPRINT:tcpCurrEstab:AVERAGE:TCP Established Connections  \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  netstat1 => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Net Statistics Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host NET Statistics Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host NET Statistics Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host NET Statistics Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:udpInDatagrams=$RRD:udpInDatagrams:AVERAGE},
            qq{DEF:udpOutDatagrams=$RRD:udpOutDatagrams:AVERAGE},
            qq{DEF:udpInErrors=$RRD:udpInErrors:AVERAGE},
            qq{LINE3:udpInDatagrams#00FF00:UDP In Datagrams},
            qq{LINE1:udpOutDatagrams#0000FF:UDP Out Datagrams},
            qq{LINE2:udpInErrors#FF0000:UDP In Errors},
            qq{COMMENT:\\n},
            qq{GPRINT:udpInDatagrams:LAST:UDP In Datagrams       \\: %5.1lf%s (cur) },
            qq{GPRINT:udpInDatagrams:MAX: \\: %5.1lf%s (max) },
            qq{GPRINT:udpInDatagrams:MIN: \\: %5.1lf%s (min) },
            qq{GPRINT:udpInDatagrams:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:udpOutDatagrams:LAST:UDP Out Datagrams      \\: %5.1lf%s (cur) },
            qq{GPRINT:udpOutDatagrams:MAX: \\: %5.1lf%s (max) },
            qq{GPRINT:udpOutDatagrams:MIN: \\: %5.1lf%s (min) },
            qq{GPRINT:udpOutDatagrams:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:udpInErrors:LAST:UDP In Errors          \\: %5.1lf%s (cur) },
            qq{GPRINT:udpInErrors:MAX: \\: %5.1lf%s (max) },
            qq{GPRINT:udpInErrors:MIN: \\: %5.1lf%s (min) },
            qq{GPRINT:udpInErrors:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  netstat => {
    hourly => {
        start_sec => "e-48h",
        title => "$host TCP Statistics Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host TCP Statistics Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host TCP Statistics Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host TCP Statistics Last 576 Days",
    },
    yaxis => "bits/second",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        return [
            qq{DEF:tcpInInorderBytes=$RRD:tcpInInorderBytes:AVERAGE},
            qq{DEF:tcpOutDataBytes=$RRD:tcpOutDataBytes:AVERAGE},
            qq{DEF:tcpRetransBytes=$RRD:tcpRetransBytes:AVERAGE},
	    qq{CDEF:tcpInInorderBits=tcpInInorderBytes,8,*},
	    qq{CDEF:tcpOutDataBits=tcpOutDataBytes,8,*},
	    qq{CDEF:tcpRetransBits=tcpRetransBytes,8,*},
            qq{LINE3:tcpInInorderBits#00FF00:In },
            qq{LINE1:tcpOutDataBits#0000FF:Out },
            qq{LINE2:tcpRetransBits#FF0000:Retrans },
            qq{COMMENT:\\n},
            qq{GPRINT:tcpInInorderBits:LAST:In       \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpInInorderBits:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpInInorderBits:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpInInorderBits:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:tcpOutDataBits:LAST:Out      \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpOutDataBits:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpOutDataBits:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpOutDataBits:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:tcpRetransBits:LAST:Retrans  \\: %5.1lf%s (cur)},
            qq{GPRINT:tcpRetransBits:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:tcpRetransBits:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:tcpRetransBits:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  sendmail => {
    hourly => {
        start_sec => "e-48h",
        title => "$host sendmail Utilization Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Utilization Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Utilization Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Utilization Last 576 Days",
    },
    yaxis => "Stats",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/^.*sendmail\./\,/;
        $part =~ s/\.rrd//;
        $part =~ s/\,//g;


        return [
            qq{DEF:$p}   . qq{IN=$RRD:msgsfr:AVERAGE},
            qq{CDEF:$p}  . qq{IN1=$p} . qq{IN,300,*},
            qq{LINE1:$p} . qq{IN1#$color:$part IN},
            qq{COMMENT:\\n},
            qq{GPRINT:$p} . qq{IN1:LAST:$part in   %5.1lf (cur)},
            qq{GPRINT:$p} . qq{IN1:MAX: \\: %5.1lf (max)},
            qq{GPRINT:$p} . qq{IN1:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p} . qq{IN1:AVERAGE: \\: %5.1lf (avg)\\n},

            qq{DEF:$p}   . qq{OUT=$RRD:msgsto:AVERAGE},
            qq{CDEF:$p}  . qq{OUT1=$p} . qq{OUT,300,*},
            qq{LINE2:$p} . qq{OUT1#} . &color() . qq{:$part OUT},
            qq{COMMENT:\\n},
            qq{GPRINT:$p} . qq{OUT1:LAST:$part out  %5.1lf (cur)},
            qq{GPRINT:$p} . qq{OUT1:MAX: \\: %5.1lf (max)},
            qq{GPRINT:$p} . qq{OUT1:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p} . qq{OUT1:AVERAGE: \\: %5.1lf (avg)\\n},
        ]

       # return [
            #qq{DEF:IN1=$host.sendmail.$part.rrd:msgsfr:AVERAGE},
            #qq{CDEF:IN1=IN,300,*},
            #qq{DEF:OUT=$host.sendmail.$part.rrd:msgsto:AVERAGE},
            #qq{CDEF:OUT1=OUT,300,*},
            #qq{LINE2:IN1#FF0000:Inbound Mail Items},
            #qq{LINE2:OUT1#0000FF:Outbound Mail Items},
        #]
    },
  },

  users => {
    hourly => {
        start_sec => "e-48h",
        title => "$host # of Users and Processes Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host # of Users and Processes Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host # of Users and Processes Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host # of Users and Processes Last 576 Days",
    },
    yaxis => "#",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

	return [
            qq{DEF:p=$host.procs.rrd:la:AVERAGE},
            qq{DEF:u=$host.users.rrd:la:AVERAGE},
            qq{AREA:p#00CC00:Processes},
            qq{LINE1:u#000099:Users},
            qq{COMMENT:\\n},
            qq{GPRINT:p:LAST:Processes   \\: %5.1lf%s (cur)},
            qq{GPRINT:p:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:p:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:p:AVERAGE: \\: %5.1lf%s (avg)\\n},
            qq{GPRINT:u:LAST:Users       \\: %5.1lf%s (cur)},
            qq{GPRINT:u:MAX: \\: %5.1lf%s (max)},
            qq{GPRINT:u:MIN: \\: %5.1lf%s (min)},
            qq{GPRINT:u:AVERAGE: \\: %5.1lf%s (avg)\\n},
        ]
    },
  },

  temperature => {
    hourly => {
        start_sec => "e-48h",
        title => "$host Temperatures Last 48 Hours",
    },
    daily => {
        start_sec => "e-12d",
        title => "$host Temperatures Last 12 Days",
    },
    weekly => {
        start_sec => "e-48d",
        title => "$host Temperatures Last 48 Days",
    },
    monthly => {
        start_sec => "e-576D",
        title => "$host Temperatures Last 576 Days",
    },
    yaxis => "Degrees Celsius",
    lines => sub {
        my ($RRD,$service,$graph,$p,$color)=@_;

        my($part) = $RRD;

        $part =~ s/^.*temperature,/,/;
        $part =~ s/\.rrd//;
        $part =~ s/\,/\//g;
        $part =~ s/_/ /g;
        $part =~ s/^\///g;

        # Pad with spaces or line feed for GPRINT below
        my $padding = '                  ';
        my $comment = '\\n';
        if (length($part) <= 14) {
            $padding = substr($padding,0,14-length($part));
            $comment = '';
        }

        return [
            qq{DEF:$p=$RRD:temperature:AVERAGE},
            qq{LINE2:$p#$color:$part},
            qq{COMMENT:$comment},
            qq{GPRINT:$p:LAST:$padding \\: %5.1lf (cur)},
            qq{GPRINT:$p:MAX: \\: %5.1lf (max)},
            qq{GPRINT:$p:MIN: \\: %5.1lf (min)},
            qq{GPRINT:$p:AVERAGE: \\: %5.1lf (avg)\\n},
        ]
    },
  },

); # end of graph_c hash

print TMP "start of SWITCH: $service\n" if $DEBUG;
SWITCH : {
    if ($service eq "conn" ) { $service="tcp"; last SWITCH; };
    if ($service eq "fping" ) { $service="tcp"; last SWITCH; };
    if ($service eq "http" ) { $service="tcp"; last SWITCH; };
    if ($service eq "cpu" ) { $service="la"; last SWITCH; };
}

my $len;
my $bits;
my $pic;

if ($service eq "larrd" or not exists $graph_c{$service}) {
  # think this is displayed if we have nothing to graph
    open(GRAPH,"$ENV{'LARRDTMP'}/onebyone.png") or die $!;
    binmode(GRAPH);
    while ($len = read(GRAPH,$bits,8192)) {
        $pic .= $bits;
    }
    close(GRAPH);
    print $q->header(-type=>'image/png',-expires=>'+5m');
    print $pic;
    exit;
}

# Then we are just printing a graph out
if($graph) {
   print TMP "start of second SWITCH: $graph\n" if $DEBUG;

    SWITCH : {
        if ($service eq "cpu" ) { $service="la"; last SWITCH; }
        if ($service eq "http" ) { $service="tcp"; $single="http"; last SWITCH; }
        if ($service eq "ftp" ) { $service="tcp"; $single="ftp"; last SWITCH; }
        if ($service eq "conn" ) { $service="tcp"; $single="conn"; last SWITCH; }
        if ($service eq "fping" ) { $service="tcp"; $single="fping"; last SWITCH ; }
        if ($service eq "ssh" ) { $service="tcp"; $single="ssh"; last SWITCH; }
        if ($service eq "telnet" ) { $service="tcp"; $single="telnet"; last SWITCH; }
        if ($service eq "nntp" ) { $service="tcp"; $single="nntp"; last SWITCH; }
        if ($service eq "pop" ) { $service="tcp"; $single="pop"; last SWITCH; }
        if ($service eq "pop-2" ) { $service="tcp"; $single="pop-2"; last SWITCH; }
        if ($service eq "pop-3" ) { $service="tcp"; $single="pop-3"; last SWITCH; }
        if ($service eq "pop2" ) { $service="tcp"; $single="pop2"; last SWITCH; }
        if ($service eq "pop3" ) { $service="tcp"; $single="pop3"; last SWITCH; }
        if ($service eq "smtp" ) { $service="tcp"; $single="smtp"; last SWITCH; }
        if ($service eq "imap" ) { $service="tcp"; $single="imap"; last SWITCH; }
    }

    if ($service eq "larrd" or not exists $graph_c{$service}) {
  # think this is displayed if we have nothing to graph
        open(GRAPH,"$ENV{'LARRDTMP'}/onebyone.png") or die $!;
        binmode(GRAPH);
        while ($len = read(GRAPH,$bits,8192)) {
            $pic .= $bits;
        }
        close(GRAPH);
        print $q->header(-type=>'image/png',-expires=>'+5m');
        print $pic;
        exit;
    }

    $graph = "hourly" if $graph !~ /hourly|daily|weekly|monthly/;

# TODO make this a config
     my ($RRD_graph) = "$ENV{'LARRDTMP'}/$host.$service$single.$graph$disk_part.png";

    RRDs::graph($RRD_graph,"-s",$graph_c{$service}{$graph}{'start_sec'},
        "--title",$graph_c{$service}{$graph}{'title'},"-w576",
        "-v",$graph_c{$service}{'yaxis'},"-a","PNG","--lazy",
        &defs($host,$service,$graph,$disk_part),
        qq{COMMENT:$date\r});

    my($RRDerr) = RRDs::error;
    die $RRDerr if $RRDerr;

    open(GRAPH,"<$RRD_graph") or die $!;
    binmode(GRAPH);

    while ($len = read(GRAPH,$bits,8192)) {
        $pic .= $bits;
    }

    close(GRAPH);

    print $q->header(-type=>'image/png',-expires=>'+5m');
    print $pic;

} else {

    print $q->header;
    #print qq{<html><body bgcolor="#FFFFFF"><center>};
print qq{<html><body BGCOLOR="green" BACKGROUND="/bb/gifs/bkg-green.gif" TEXT="#D8D8BF" LINK="#00FFAA" VLINK="#FFFF44"><center>};

    if ($service eq "la") {
      print qq{<h1>CPU trends for $host</h1><table>};
    } else {
        print qq{<h1>$service trends for $host</h1><table>};
    }


    foreach my $graph ("hourly","daily","weekly","monthly") {
        print qq{<tr><th>$graph</th></tr>};
        print qq{<tr><td><IMG SRC="$cgiurl?host=$host&service=$service&graph=$graph&disk_part=$disk_part" ALT="larrd is accumulating"><BR></td></tr>};

    }

    print qq{</table><br></center>};
    print qq{larrd is proud to be built on };
    print qq{<A HREF="http://ee-staff.ethz.ch/~oetiker/webtools/rrdtool/">};
    print qq{RRDtool</A>};
    print qq{&nbsp;&nbsp;};
    print qq{<A HREF="/bb/">};
    print qq{Back to Main BigBrother Page</A>};
}


sub defs {

    my($host,$service,$graph) = @_;
    my(@defs);
    my($p)=0;

    print TMP "in defs\n" if $DEBUG;
    print TMP "host: $host\n" if $DEBUG;
    print TMP "service: $service\n" if $DEBUG;
    print TMP "graph: $graph\n" if $DEBUG;

    # TODO get rid of this.
    chdir($ENV{'BBRRDS'}) or die "Could not change directory to the BBRRDS ($ENV{'BBRRDS'}) directory : $!";

    &color(1);

    # TODO this is very unfast!
    # FIXED: removed the * from the end of $service
    #  disk stuff was being returned for dl service
    my $svc = $service;
    $svc =~ s/[0-9]$//;
    opendir(DIR,$ENV{'BBRRDS'});
    my @rrds = sort grep { /^$host\.$svc/ } readdir(DIR);
    closedir(DIR);

    if (("$service" eq "disk1") or ("$service" eq "disk")) {
       my @disk_array;
       my %disk_hash;

       foreach my $RRD (@rrds) {
         my @items = split /\.disk/, $RRD;
         next if ! defined $items[1];
         my $disk_name = $items[1];
#         substr($disk_name, -4) = "";
         $disk_name = "/" if ($disk_name eq ",root");
         $disk_name =~ s/\,/\//g;
         push @disk_array, $disk_name;
         $disk_hash{$disk_name} = $RRD;
       }

       my @sub_array = sort @disk_array;
       my ($a, $b) = split (/\.\./, $disk_part);
       @sub_array = @sub_array[$a .. $b];


       foreach my $disk ( @sub_array ) {
         print TMP "in sub array: $disk\n" if $DEBUG;
         last if $disk eq "";
         @defs = (@defs,@{&{$graph_c{$service}{'lines'}}($disk_hash{$disk},$service,$graph,$p++,&color())});
         $disk = "";
       }

    } else {
      foreach my $RRD (@rrds) {
        print TMP "$RRD $single\n" if $DEBUG;
        if ($single) { unless ($RRD =~ /$single/) { next; } }
          @defs = (@defs,@{&{$graph_c{$service}{'lines'}}($RRD,$service,$graph,$p++,&color() )});
        }
    }

    print TMP "@defs\n" if $DEBUG;
    return @defs;
}


sub color {

    my $reset = shift;

    my(@color) = qw{0000FF FF0000 00CC00 FF00FF 555555 880000 000088 008800 008888 888888 880088 FFFF00 888800 00FFFF 00FF00 AA8800 AAAAAA DD8833 DDCC33 8888FF 5555AA B428D3 FF5555 DDDDDD AAFFAA AAFFFF FFAAFF FFAA55 55AAFF AA55FF};
    # rainbow colors
    #my(@color) = qw{AAAAAA FF0000 DD8833 DDCC33 00FF00 8888FF 5555AA B428D3 000000 777777 };

    if($reset) {
        $p=0;
    }

#fix added by CW 11-3-2000 <chris.warburton@rs-components.com>
    if($p >= scalar @color) {
        $p=0;
    }

    return $color[$p++];

}
